#
#  Copyright 1996-2011 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::db - Checksum database module
#
package Security::db;

use Security::util::SHA1;
use Exporter;
use DB_File;
use Fcntl;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( &init() &verify() &print() &destroy() );

our (%checksum, %badchecksums, %missingchecksumfiles);

sub init {
	print ( STDERR  "In Security::db::init()\n") if ( $main::opts{'d'} );

	tie %checksum, 'DB_File', $main::checksum_file, 
		O_CREAT|O_RDWR, 00600, $DB_HASH;

	&main::read_config('checksumfiles');
}

sub verify {
	print ( STDERR  "In Security::db::verify()\n") if ( $main::opts{'d'} );

	my ($file) = @_;
	my ($currsum);

	grep (/^$main::format$/ && ($currsum=$2), &Security::util::SHA1::check($file));
	if ( defined $checksum{$file} && $currsum ne $checksum{$file} ) {
		$badchecksums{$file} = "old ($checksum{$file}) new ($currsum)";
	}
	$checksum{$file} = $currsum;
}

sub check {
	print ( STDERR  "In Security::db::check()\n") if ( $main::opts{'d'} );

	foreach my $file ( @main::checksumfiles ) {
		if ( -f $file ) {
			&verify($file);
		} else {
			$missingchecksumfiles{$file} = 1;
		}
	}
}

sub print {
	print ( STDERR  "In Security::db::print()\n") if ( $main::opts{'d'} );

	&main::print_data ("\n\nchecking checksum mismatch:\n", %badchecksums);
	&main::print_data ("\n\nMissing checksum files:\n", %missingchecksumfiles);
}

sub destroy {
	print ( STDERR  "In Security::db::destroy()\n") if ( $main::opts{'d'} );

	untie %checksum;
}

1;

__END__

=head1 NAME

Security::db - Store file checksums.

=head1 SYNOPSIS

Security::db::verify($file);

=head1 DESCRIPTION

Compares a file's current checksum with one that has been saved from a
previous run.

=head1 API

This module implements the default API:

=over 4

=item C<init()>

Calls C<read_config (checksumfiles)> to get the list of additional files
we should be tracking the checksum of.  Will tie an internal hash variable
to the F<.checksum.db> file, as defined in C<$main::checksum_file>, so we
can get the old checksums.  If F<.checksum.db> does not exist, it will
create it with mode I<600>.

=item C<check()>

Goes through the list of files in C<@main::checksumfiles> and verifies
their checksum.  

=item C<print()>

Calls C<print_data ()> to print out the list of files that had a
checksum mismatch.

=item C<destroy()>

Untie's the internal hash variable and the F<.checksum.db> file.

=back

It also provides the following API calls for other modules to use.

=over 4

=item C<verify($file)>

Provides a comparison routine to check the current checksum for $file with
the stored one in F<.checksum.db>.

=back

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<checksumfiles>

The list of additional files to checksum.

=back

=head1 HISTORY

The checksumming of non special files was suggeded by a former coworker at
UIUC.

=head1 SEE ALSO

L<tripwire(8)>

L<DB_File>

L<perl>

=cut

