#
#  Copyright 1996-2011 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::mail - mail config file control module
#
package Security::mail;

use Security::util::stat;
use Security::util::users;
use Exporter;
use POSIX;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw(&init() &check() &print());

our (%mailowner, %mailgroup, %mailworld);

sub init {
	print ( STDERR  "In Security::mail::init()\n") if ( $main::opts{'d'} );

	&main::read_config('mailfiles');
	&main::read_config('ignoremailfiles');
}

sub check {
	print ( STDERR  "In Security::mail::check()\n") if ( $main::opts{'d'} );

	foreach my $name ( Security::util::users::get('name') ) {
		my $dir = Security::util::users::get('name', $name, 'dir');
		my $uid = Security::util::users::get('name', $name, 'uid');
		if ( !grep (/^\Q$name\E/, @main::ignoremailfiles) && -d $dir ) {
			foreach my $file ( @main::mailfiles ) {
				if ( -f $dir . '/' . $file ) {

					# Use stat vs lstat, just in case someone has their
					# .forward file as a symlink to a different file.
					my ($dev,$ino,$mode,$nlink,$u,$g,$rdev,$curbytes) = 
						&Security::util::stat::get_stat($dir, 'stat');

					# If the home dir does not match this account
					# flag it, since we have a file in the 
					# mailfiles list.
					$mailowner{$dir} = 1
						if ( $uid != $u );

					# If the home dir is group or world writeable
					# that means they could have tweaked the 
					# file as well.
					$mailgroup{$dir} = 1
						if ( $mode & &S_IWGRP() );

					$mailworld{$dir} = 1
						if ( $mode & &S_IWOTH() );

					($dev,$ino,$mode,$nlink,$u,$g,$rdev,$curbytes) = 
						&Security::util::stat::get_stat($dir . '/' . $file, 'stat');

					# If the file itself is not owned by this user,
					# flag this as a problem.
					$mailowner{$dir . '/' . $file} = 1
						if ( $uid != $u );

					# If the file itself is group or world writeable
					# that means some one else could have tweaked the file
					$mailgroup{$dir . '/' . $file} = 1
						if ( $mode & &S_IWGRP() );

					$mailworld{$dir . '/' . $file} = 1
						if ( $mode & &S_IWOTH() );
				}
			}
		}
	}
}

sub print {
	print ( STDERR  "In Security::mail::print()\n") if ( $main::opts{'d'} );

	&main::print_data ("\n\nchecking for mail files with improper ownership:\n", %mailowner);
	&main::print_data ("\n\nchecking for mail files with group writeable mode:\n", %mailgroup);
	&main::print_data ("\n\nchecking for mail files with world writeable mode:\n", %mailworld);
}

1;

__END__

=head1 NAME

Security::mail - mail file check

=head1 SYNOPSIS

Security::mail::init()

Security::mail::check()

Security::mail::print()

=head1 DESCRIPTION

Checks the file modes of files involved in the delivery of mail.

=head1 API

This module implements the default API:

=over 4

=item C<init()>

Calls C<read_config (mailfiles)> to get the list of files to look for
in a user's home directory.

Calls C<read_config (ignoremailfiles)> to get the list of accounts to
skip checking for the mailfiles.

=item C<check()>

Goes through each account on the system (via C<Security::util::users-:get()>),
and check to see if they have any file listed in C<@main::mailfiles>).
If so, then call C<Security::util::stat::get_stat($file, 'stat')> to check the
owner and mode of the file.  It will also check to see if the home directory
has the correct ownership and mode.

=item C<print()>

Calls C<print_data ()> to print out the list of mail files with the wrong
owner.

Calls C<print_data ()> to print out the list of mail files that are world
writable.

Calls C<print_data ()> to print out the list of mail files that are group
writable.

=back

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<mailfiles>

The list of files to check to make sure the owner, group and mode are set
correctly.

=item C<ignoremailfiles>

The list of accounts to ignore problems with mail files.  Usually is the list
of accounts that have the same home directory, most commonly F</>.

=back

=head1 HISTORY

The idea for this came from error messages from C<sendmail(1)>

=head1 SEE ALSO

L<Security::util::stat>

L<Security::util::users>

L<sendmail(1)>

L<procmail(1)>

L<foward(5)>

=cut
